#!/usr/bin/env python

"""
    This program will read a file and calculate its digest
    value using the Whirlpool hashing algorithm, version 2.
    
    Whirlpool is a hash function designed by Vincent Rijmen 
    and Paulo Barreto that operates on messages less than 
    2^256 bits in length, and produces a message digest of 
    512 bits.
    
    Whirlpool has been selected for the NESSIE (EU) portfolio 
    of cryptographic primitives. The International Organization
    for Standardization (ISO) has decided to include the final 
    version of Whirlpool in the revised ISO/IEC 10118-3:2003(E)
    standard. 
    
    The Whirlpool algorithm has undergone two revisions since 
    its original 2000 specification (v0).  In the first revision 
    in 2001 (v1), the s-box was changed from a randomly 
    generated one with good cryptographic properties to one 
    which has better cryptographic properties and is easier to 
    implement in hardware.  Then, in 2003, the diffusion matrix 
    was changed in the second revision (v2).
    
    This program makes use of the 2003 version, aka v2.
    
    Whirlpool is not (and will never be) patented. It may be 
    used free of charge for any purpose. The reference 
    implementations are in the public domain.
    
    For more information, see
      http://paginas.terra.com.br/informatica/paulobarreto/WhirlpoolPage.html
        
    Larry Bugbee
    May 2005

"""

from whirlpool import *
import sys

#---------------------------------------------------------------
# utilities

def bytestr2hex(bs):
    h = ''
    for c in bs:
        h = h + ('0'+hex(ord(c))[2:])[-2:]
    return h

#---------------------------------------------------------------

if len(sys.argv) != 2:
    print '\n  Usage:'
    print '\n      python  %s  <<filename>>\n' % sys.argv[0]
    sys.exit()

filename = sys.argv[1]
f = open(filename, 'rb')
wp = whirlpool()
while 1:
    print '.',
    sys.stdout.flush()
    buf = f.read(200000)
    if buf:
        wp.update(buf)
    else:
        break
f.close()
d = wp.digest()

print
print '\nThe Whirlpool digest for "%s" is:' % filename
print '  0x%s' % bytestr2hex(d)
print

#---------------------------------------------------------------
#---------------------------------------------------------------
#---------------------------------------------------------------
