#!/usr/bin/env python

"""
    If you are running Python 2.4 or lower and you haven't 
    done so already, install ctypes.  (ctypes will be standard
    with Python 2.5)
    
    Be sure to designate the install_dir below.  ...or you may 
    delete the reference to data_files below (approx line 56) 
    and move the whirlpool.pth file manually.  Your choice.
    
    To compile with MinGW:
        python setup.py build -cmingw32 [install]
    otherwise:
        python setup.py build [install]

    Larry Bugbee
    May 2005
    rev August 2006

"""

from distutils.core import setup, Extension
import sys


# Where should we copy the data file "whirlpool.pth"?  Depending
# on your system, you may need to modify appropriately.
if sys.platform == 'win32':
    install_dir = 'lib/site-packages'           # windows
elif sys.platform in ['darwin', 'posix', 'linux2']:
    install_dir = 'lib/python2.4/site-packages' # macosx and linux
else:
    raise 'mod setup.py to recognize this platform type.'


whirly = Extension(name = 'whirlpool._whirlpool',
                   sources = ['whirlpool/Whirlpool.c',
                              'whirlpool/Whirlpool-additions.c',
                             ],
                   export_symbols = ['NESSIEinit',
                                     'NESSIEadd',
                                     'NESSIEfinalize',
                                     'getNESSIEstructSize',
                                    ],
                  )

setup(name = 'Whirlpool',
      version = '2',            # the 2003 version
      description = 'Whirlpool: a 512-bit hashing algorithm',
      long_description = """
        Algorithm and C reference code created by
        Paulo S.L.M. Barreto and Vincent Rijmen.
        ctypes wrapper written by Larry Bugbee.
      """,
      author = 'Larry Bugbee',
      packages = [],
      data_files  = [(install_dir, ('whirlpool/whirlpool.pth',))],
      py_modules  = ['whirlpool.whirlpool'],
      ext_modules = [whirly],
     )

