#!/usr/bin/env python

"""
    Simple test of Python/ctypes/C version of Whirlpool using
    test vectors obtained from various 'independent' sources.
    
    The Whirlpool algorithm has undergone two revisions since 
    its original 2000 specification (v0).  In the first revision 
    in 2001 (v1), the s-box was changed from a randomly 
    generated one with good cryptographic properties to one 
    which has better cryptographic properties and is easier to 
    implement in hardware.  Then, in 2003, the diffusion matrix 
    was changed in the second revision (v2).
    
    This is the 2003 version, aka v2.
    
    Larry Bugbee
    May 2005
    rev August 2006

"""

from whirlpool import Whirlpool

filename  = 'Whirlpool-v2-TVpy.txt'
testlimit = 0      # set to 0 to run all tests

#---------------------------------------------------------------
# utilities

def bytestr2hex(bs):
    h = ''
    for c in bs:
        h = h + ('0'+hex(ord(c))[2:])[-2:]
    return h

def hex2bytestr(h):
    bs = ''
    if h[-1:] == 'L':
        h = h[:-1]
    if h[:2] == '0x':
        h = h[2:]
    if len(h) % 2:
        h = '0'+h
    while h:
        bs += chr(eval('0x'+h[:2]))
        h = h[2:]
    return bs

#---------------------------------------------------------------

def hashMe(msg, hash):
    wp = Whirlpool()        # or...  wp = Whirlpool.new()
    wp.update(msg)          # may repeat if have many chunks 
    d = wp.digest()         # returns final digest (do not 
                            #   continue after reading the 
                            #   digest value)
    return d

#---------------------------------------------------------------

template = """\
      predict %s
      actual  %s"""

f = open(filename)
n = 0
for line in f:
    if not line:  break
    if not line.strip(): continue
    if line[:1] == '#':  continue
    if testlimit:
        n += 1
        if n > testlimit: break
    msg, hash = eval(line)
    if len(msg) > 54: 
        msgstr = '"%s...%s"' % (msg[:47], msg[-4:])
    else:
        msgstr = '"%-55s' % (msg[:54] + '"')
    if msg[:2] == '0x':
        m = hex2bytestr(msg)
    else:
        m = msg
    h = hex2bytestr(hash)
    d = hashMe(m,h)
    if d != h:
        print '  *** test %s failed *** ' % msgstr
        print template % (bytestr2hex(h), bytestr2hex(d))
    else:
        print '      test %s passed ' % msgstr

f.close()

#---------------------------------------------------------------
#---------------------------------------------------------------
#---------------------------------------------------------------
